/**
 * Created by John on 11/17/2016.
 *
 * Wrapper and common routines for handling RabbitMQ queues
 */

function RabbitHelper(winston, config) {
    this.rabbit = require("rabbot");
    this.winston = winston;
    this.logger = (winston) ? winston.loggers.get('transaction') : console;
    if (!this.logger.debug) {
        this.logger.debug = console.log;
    }
    var self = this;
    // it can make a lot of sense to share topology definition across
    // services that will be using the same topology to avoid
    // scenarios where you have race conditions around when
    // exchanges, queues or bindings are in place
    require("../config/RabbitTopology.js")(this.rabbit, null, config)
        .then(function() {
            self.logger.debug("Rabbit Topology Loaded...");
        });

    this.rabbit.on("unreachable", function() {
        self.logger.error("Unable to connect to RabbitMQ :(");
        process.exit();
    });
}


/**
 * Genericly publishes to a RabbitMQ queue.
 *
 * @param {String} key (asn or po)
 * @param {Object} data ASN
 * @returns {Array.Promise} Array of Promise responses
 */
RabbitHelper.prototype.publishGeneric = function(key, data) {
    var i;
    var self = this;
    var promises = [];
    var send = function(x, idx, max) {
        return new Promise(function(resolve, reject){
            var direction = key;
            self.rabbit.publish("topic-fresenius-x", {
                routingKey: direction,
                type: direction,
                body: x, //Must be set to true for onReturned to receive unqueued message
                persistent: true
            }).then(function() {
                self.logger.debug("published " + key + " message", x);
                var out = {
                    msg: x,
                    idx: idx,
                    total: max
                };
                resolve(out);
            }).catch(function(err){
                reject(err);
            });
        });
    };

    if (Array.isArray(data)) {
        for (i = 0; i < data.length; i++) {
            promises.push(send(data[i], i, data.length));
        }
    } else {
        promises.push(send(data, 0, 1));
    }

    return Promise.all(promises);

};

RabbitHelper.prototype.publishAsn = function(msg, cb) {
    var self = this;
    var direction = "asn";
    this.publishGeneric(direction, msg).then(function(data){
        if (cb) cb(null, data);
    }).catch(function(err){
        if (cb) cb(err, null);
    });
};

RabbitHelper.prototype.publishPO = function(msg, cb) {
    var self = this;
    var direction = "po";
    this.publishGeneric(direction, msg).then(function(data){
        if (cb) cb(null, data);
    }).catch(function(err){
        if (cb) cb(err, null);
    });
};

RabbitHelper.prototype.cleanUp = function(cb) {
    this.rabbit.shutdown().then(function(){
        if (cb) cb(null, null);
    }).catch(function(err){
        if (cb) cb(err, null);
        else throw err;
    });
};

module.exports = RabbitHelper;